﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Windows.Controls;

namespace SilverlightScroller
{
    public partial class ScrollerControl: UserControl
    {
        private AnimationHelperControl _helper;

        public ScrollerControl()
        {
            // Required to initialize variables
            InitializeComponent();

            InitializeAnimationHelper();
        }

        #region Private Properties
        private double _itemWidth;
        internal double ItemWidth
        {
            get
            {
                if (_itemWidth == default(double))
                {
                    _itemWidth = theScrollViewer.GetLogicalChildrenBreadthFirst().OfType<WrapPanel>().FirstOrDefault().ItemWidth;
                }
                return _itemWidth;
            }
        }
        #endregion

        #region Custom Dependency Properties
        
        /// <summary>
        ///     Scroll Time per click in Milliseconds - int naturally defaults to zero so if not set will essentially have no animation
        /// </summary>
        public int ScrollTime
        {
            get { return (int) GetValue(ScrollTimeProperty); }
            set { SetValue(ScrollTimeProperty, value); }
        }

        public static readonly DependencyProperty ScrollTimeProperty =
            DependencyProperty.Register("ScrollTime", typeof(int), typeof(ScrollerControl),
                new PropertyMetadata(new PropertyChangedCallback(OnScrollTimeChanged)));

        private static void OnScrollTimeChanged(DependencyObject sender, DependencyPropertyChangedEventArgs args)
        {
            if (args.OldValue != args.NewValue)
            {
                (sender as ScrollerControl).ScrollTime = (int) args.NewValue;
            }
        }
        
        #endregion
        
        #region Event handlers
        
        private void PreviousButton_Click(object sender, RoutedEventArgs e)
        {
            //  do not scroll if the offset is not at least the width of a single item
            if (theScrollViewer.HorizontalOffset >= ItemWidth)
            {
                // build and fire the animation setting the OnCompleted event to synch the buttons
                _helper.AnimateDoubleTo(ScrollTime,
                    "(AnimationHelperControl.AnimationValue)",
                    GetNewHorizontalOffset(-ItemWidth),
                    (s, args) => { SynchButtonDisplay(); });
            }
        }

        private void NextButton_Click(object sender, RoutedEventArgs e)
        {
            if (GetScrollableWidthRemaining() >= ItemWidth)
            {
                // build and fire the animation setting the OnCompleted to synch the buttons
                _helper.AnimateDoubleTo(ScrollTime,
                    "(AnimationHelperControl.AnimationValue)",
                    GetNewHorizontalOffset(ItemWidth),
                    (s, args) => { SynchButtonDisplay(); });
            }
        }
        
        #endregion
        
        #region Private methods

        private double GetNewHorizontalOffset(double changeAmount)
        {
            // computes the new HorizontalOffset assuring that we only move in full item width increments    
            return Math.Round((theScrollViewer.HorizontalOffset + changeAmount) / ItemWidth) * ItemWidth;
        }

        private double GetScrollableWidthRemaining()
        {
            return (theScrollViewer.ScrollableWidth - theScrollViewer.HorizontalOffset);
        }
        
        private void InitializeAnimationHelper()
        {
            _helper = new AnimationHelperControl();

            // here we hook the AnimationValueChange from our AnimationControlHelper
            //   so that we can animate the Scroll via a DoubleAnimation 
            _helper.AnimationValueChange += (sender, args) =>
            {
                theScrollViewer.ScrollToHorizontalOffset(args.Value);
            };
        }

        private void SynchButtonDisplay()
        {
            NextButton.IsEnabled = (GetScrollableWidthRemaining() >= ItemWidth);
            PreviousButton.IsEnabled = (theScrollViewer.HorizontalOffset >= ItemWidth);
        }

        #endregion
    }
}